@@ -6,19 +6,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "crypto.h"
-#include "log.h"
-#include "util.h"
+#include "torlog.h"
+#include "../common/util.h"
 #include "compat.h"
 #include <openssl/bn.h>
 #include <openssl/rsa.h>
 
-int main(int c, char **v)
+int
+main(int c, char **v)
 {
   crypto_pk_env_t *env;
   char *str;
   RSA *rsa;
   int wantdigest=0;
   int fname_idx;
+  char *fname=NULL;
   init_logging();
 
   if (c < 2) {
@@ -29,7 +31,7 @@ int main(int c, char **v)
     return 1;
   }
 
-  if (crypto_global_init(0)) {
+  if (crypto_global_init(0, NULL, NULL)) {
     fprintf(stderr, "Couldn't initialize crypto library.\n");
     return 1;
   }
@@ -46,7 +48,9 @@ int main(int c, char **v)
     fname_idx = 1;
   }
 
-  str = read_file_to_str(v[fname_idx], 0, NULL);
+  fname = expand_filename(v[fname_idx]);
+  str = read_file_to_str(fname, 0, NULL);
+  tor_free(fname);
   if (!str) {
     fprintf(stderr, "Couldn't read %s\n", v[fname_idx]);
     return 1;
