@@ -13,6 +13,7 @@
 
 #include <openssl/evp.h>
 #include <openssl/pem.h>
+#include <openssl/rsa.h>
 #include <openssl/objects.h>
 #include <openssl/obj_mac.h>
 #include <openssl/err.h>
@@ -27,8 +28,8 @@
 #define CRYPTO_PRIVATE
 
 #include "compat.h"
-#include "util.h"
-#include "log.h"
+#include "../common/util.h"
+#include "../common/torlog.h"
 #include "crypto.h"
 #include "address.h"
 
@@ -63,7 +64,6 @@ show_help(void)
           "[-c certificate_file]\n"
           "        [-m lifetime_in_months] [-a address:port] "
           "[--passphrase-fd <fd>]\n");
-
 }
 
 /* XXXX copied from crypto.c */
@@ -218,6 +218,20 @@ parse_commandline(int argc, char **argv)
   return 0;
 }
 
+static RSA *
+generate_key(int bits)
+{
+  RSA *rsa = NULL;
+  crypto_pk_env_t *env = crypto_new_pk_env();
+  if (crypto_pk_generate_key_with_bits(env,bits)<0)
+    goto done;
+  rsa = _crypto_pk_env_get_rsa(env);
+  rsa = RSAPrivateKey_dup(rsa);
+ done:
+  crypto_free_pk_env(env);
+  return rsa;
+}
+
 /** Try to read the identity key from <b>identity_key_file</b>.  If no such
  * file exists and create_identity_key is set, make a new identity key and
  * store it.  Return 0 on success, nonzero on failure.
@@ -238,7 +252,7 @@ load_identity_key(void)
     }
     log_notice(LD_GENERAL, "Generating %d-bit RSA identity key.",
                IDENTITY_KEY_BITS);
-    if (!(key = RSA_generate_key(IDENTITY_KEY_BITS, 65537, NULL, NULL))) {
+    if (!(key = generate_key(IDENTITY_KEY_BITS))) {
       log_err(LD_GENERAL, "Couldn't generate identity key.");
       crypto_log_errors(LOG_ERR, "Generating identity key");
       return 1;
@@ -250,7 +264,7 @@ load_identity_key(void)
     }
 
     if (!(f = start_writing_to_stdio_file(identity_key_file,
-                                          OPEN_FLAGS_REPLACE, 0400,
+                                          OPEN_FLAGS_REPLACE | O_TEXT, 0400,
                                           &open_file)))
       return 1;
 
@@ -323,7 +337,7 @@ generate_signing_key(void)
   RSA *key;
   log_notice(LD_GENERAL, "Generating %d-bit RSA signing key.",
              SIGNING_KEY_BITS);
-  if (!(key = RSA_generate_key(SIGNING_KEY_BITS, 65537, NULL, NULL))) {
+  if (!(key = generate_key(SIGNING_KEY_BITS))) {
     log_err(LD_GENERAL, "Couldn't generate signing key.");
     crypto_log_errors(LOG_ERR, "Generating signing key");
     return 1;
@@ -335,7 +349,7 @@ generate_signing_key(void)
   }
 
   if (!(f = start_writing_to_stdio_file(signing_key_file,
-                                        OPEN_FLAGS_REPLACE, 0600,
+                                        OPEN_FLAGS_REPLACE | O_TEXT, 0600,
                                         &open_file)))
     return 1;
 
@@ -393,7 +407,6 @@ get_fingerprint(EVP_PKEY *pkey, char *out)
   return r;
 }
 
-
 /** Set <b>out</b> to the hex-encoded fingerprint of <b>pkey</b>. */
 static int
 get_digest(EVP_PKEY *pkey, char *out)
@@ -487,7 +500,6 @@ generate_certificate(void)
   return 0;
 }
 
-
 /** Entry point to tor-gencert */
 int
 main(int argc, char **argv)
@@ -496,7 +508,7 @@ main(int argc, char **argv)
   init_logging();
 
   /* Don't bother using acceleration. */
-  if (crypto_global_init(0)) {
+  if (crypto_global_init(0, NULL, NULL)) {
     fprintf(stderr, "Couldn't initialize crypto library.\n");
     return 1;
   }
